@@ -1,22 +1,31 @@
 /**
  * gdns.c -- hosts resolver
- * Copyright (C) 2009-2014 by Gerardo Orellana <goaccess@prosoftcorp.com>
- * GoAccess - An Ncurses apache weblog analyzer & interactive viewer
+ *    ______      ___
+ *   / ____/___  /   | _____________  __________
+ *  / / __/ __ \/ /| |/ ___/ ___/ _ \/ ___/ ___/
+ * / /_/ / /_/ / ___ / /__/ /__/  __(__  |__  )
+ * \____/\____/_/  |_\___/\___/\___/____/____/
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License, or (at your option) any later version.
+ * The MIT License (MIT)
+ * Copyright (c) 2009-2016 Gerardo Orellana <hello @ goaccess.io>
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * A copy of the GNU General Public License is attached to this
- * source distribution for its full text.
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
  *
- * Visit http://goaccess.prosoftcorp.com for new releases.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #define _MULTI_THREADED
@@ -48,7 +57,7 @@
 #ifdef HAVE_LIBTOKYOCABINET
 #include "tcabdb.h"
 #else
-#include "glibht.h"
+#include "gkhash.h"
 #endif
 
 #include "error.h"
@@ -59,7 +68,7 @@
 GDnsThread gdns_thread;
 static GDnsQueue *gdns_queue;
 
-/* initialize queue */
+/* Initialize the queue. */
 void
 gqueue_init (GDnsQueue * q, int capacity)
 {
@@ -69,35 +78,44 @@ gqueue_init (GDnsQueue * q, int capacity)
   q->capacity = capacity;
 }
 
-/* get current size of queue */
+/* Get the current size of queue.
+ *
+ * Returns the size of the queue. */
 int
 gqueue_size (GDnsQueue * q)
 {
   return q->size;
 }
 
-/* is the queue empty */
+/* Determine if the queue is empty.
+ *
+ * Returns true if empty, otherwise false. */
 int
 gqueue_empty (GDnsQueue * q)
 {
   return q->size == 0;
 }
 
-/* is the queue full */
+/* Determine if the queue is full.
+ *
+ * Returns true if full, otherwise false. */
 int
 gqueue_full (GDnsQueue * q)
 {
   return q->size == q->capacity;
 }
 
-/* destroy queue */
+/* Free the queue. */
 void
 gqueue_destroy (GDnsQueue * q)
 {
   free (q);
 }
 
-/* add item to the queue */
+/* Add at the end of the queue a string item.
+ *
+ * If the queue is full, -1 is returned.
+ * If added to the queue, 0 is returned. */
 int
 gqueue_enqueue (GDnsQueue * q, char *item)
 {
@@ -110,6 +128,10 @@ gqueue_enqueue (GDnsQueue * q, char *item)
   return 0;
 }
 
+/* Find a string item in the queue.
+ *
+ * If the queue is empty, or the item is not in the queue, 0 is returned.
+ * If found, 1 is returned. */
 int
 gqueue_find (GDnsQueue * q, const char *item)
 {
@@ -124,7 +146,10 @@ gqueue_find (GDnsQueue * q, const char *item)
   return 0;
 }
 
-/* remove an item from the queue */
+/* Remove a string item from the head of the queue.
+ *
+ * If the queue is empty, NULL is returned.
+ * If removed, the string item is returned. */
 char *
 gqueue_dequeue (GDnsQueue * q)
 {
@@ -138,7 +163,10 @@ gqueue_dequeue (GDnsQueue * q)
   return item;
 }
 
-/* get the corresponding host given an address */
+/* Get the corresponding hostname given an IP address.
+ *
+ * On error, a string error message is returned.
+ * On success, a malloc'd hostname is returned. */
 static char *
 reverse_host (const struct sockaddr *a, socklen_t length)
 {
@@ -152,7 +180,10 @@ reverse_host (const struct sockaddr *a, socklen_t length)
   return alloc_string (gai_strerror (st));
 }
 
-/* determine if IPv4 or IPv6 and resolve */
+/* Determine if IPv4 or IPv6 and resolve.
+ *
+ * On error, NULL is returned.
+ * On success, a malloc'd hostname is returned. */
 char *
 reverse_ip (char *str)
 {
@@ -177,22 +208,22 @@ reverse_ip (char *str)
   return NULL;
 }
 
-/* producer */
+/* Producer - Resolve an IP address and add it to the queue. */
 void
 dns_resolver (char *addr)
 {
   pthread_mutex_lock (&gdns_thread.mutex);
+  /* queue is not full and the IP address is not in the queue */
   if (!gqueue_full (gdns_queue) && !gqueue_find (gdns_queue, addr)) {
-#ifndef HAVE_LIBTOKYOCABINET
-    g_hash_table_replace (ht_hostnames, g_strdup (addr), NULL);
-#endif
+    /* add the IP to the queue */
     gqueue_enqueue (gdns_queue, addr);
     pthread_cond_broadcast (&gdns_thread.not_empty);
   }
   pthread_mutex_unlock (&gdns_thread.mutex);
 }
 
-/* consumer */
+/* Consumer - Once an IP has been resolved, add it to dwithe hostnames
+ * hash structure. */
 static void
 dns_worker (void GO_UNUSED (*ptr_data))
 {
@@ -215,20 +246,19 @@ dns_worker (void GO_UNUSED (*ptr_data))
         free (host);
       break;
     }
-#ifdef HAVE_LIBTOKYOCABINET
-    tcadbput2 (ht_hostnames, ip, host);
-    free (host);
-#else
-    if (host != NULL && active_gdns)
-      g_hash_table_replace (ht_hostnames, g_strdup (ip), host);
-#endif
+
+    /* insert the corresponding IP -> hostname map */
+    if (host != NULL && active_gdns) {
+      ht_insert_hostname (ip, host);
+      free (host);
+    }
 
     pthread_cond_signal (&gdns_thread.not_full);
     pthread_mutex_unlock (&gdns_thread.mutex);
   }
 }
 
-/* init queue and dns thread */
+/* Initialize queue and dns thread */
 void
 gdns_init (void)
 {
@@ -245,23 +275,22 @@ gdns_init (void)
     FATAL ("Failed init thread mutex");
 }
 
-/* destroy queue */
+/* Destroy (free) queue */
 void
 gdns_free_queue (void)
 {
   gqueue_destroy (gdns_queue);
 }
 
-/* create a dns thread */
+/* Create a DNS thread and make it active */
 void
 gdns_thread_create (void)
 {
-  int thread;
+  int th;
 
   active_gdns = 1;
-  thread =
-    pthread_create (&(gdns_thread.thread), NULL, (void *) &dns_worker, NULL);
-  if (thread)
-    FATAL ("Return code from pthread_create(): %d", thread);
+  th = pthread_create (&(gdns_thread.thread), NULL, (void *) &dns_worker, NULL);
+  if (th)
+    FATAL ("Return code from pthread_create(): %d", th);
   pthread_detach (gdns_thread.thread);
 }
