@@ -1,22 +1,31 @@
 /**
  * sort.c -- functions related to sort functionality
- * Copyright (C) 2009-2014 by Gerardo Orellana <goaccess@prosoftcorp.com>
- * GoAccess - An Ncurses apache weblog analyzer & interactive viewer
+ *    ______      ___
+ *   / ____/___  /   | _____________  __________
+ *  / / __/ __ \/ /| |/ ___/ ___/ _ \/ ___/ ___/
+ * / /_/ / /_/ / ___ / /__/ /__/  __(__  |__  )
+ * \____/\____/_/  |_\___/\___/\___/____/____/
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License, or (at your option) any later version.
+ * The MIT License (MIT)
+ * Copyright (c) 2009-2016 Gerardo Orellana <hello @ goaccess.io>
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * A copy of the GNU General Public License is attached to this
- * source distribution for its full text.
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
  *
- * Visit http://goaccess.prosoftcorp.com for new releases.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #if HAVE_CONFIG_H
@@ -29,6 +38,7 @@
 #include <getopt.h>
 #include <errno.h>
 
+#include "error.h"
 #include "settings.h"
 #include "util.h"
 
@@ -36,21 +46,38 @@
 
 /* *INDENT-OFF* */
 const int sort_choices[][SORT_MAX_OPTS] = {
+  /* VISITORS */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* REQUESTS */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, SORT_BY_PROT, SORT_BY_MTHD, -1},
+  /* REQUESTS_STATIC */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, SORT_BY_PROT, SORT_BY_MTHD, -1},
+  /* NOT_FOUND */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, SORT_BY_PROT, SORT_BY_MTHD, -1},
+  /* HOSTS */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* OS */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* BROWSERS */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* VISIT_TIMES */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* VIRTUAL_HOSTS */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* REFERRERS */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* REFERRING_SITES */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
-#ifdef HAVE_LIBGEOIP
+  /* KEYPHRASES */
+  {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* STATUS_CODES */
+  {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+  /* REMOTE_USER */
+  {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
+#ifdef HAVE_GEOLOCATION
+  /* GEO_LOCATION */
   {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
 #endif
-  {SORT_BY_HITS, SORT_BY_VISITORS, SORT_BY_DATA, SORT_BY_BW, SORT_BY_AVGTS, SORT_BY_CUMTS, SORT_BY_MAXTS, -1},
 };
 
 static GEnum FIELD[] = {
@@ -79,17 +106,26 @@ GSort module_sort[TOTAL_MODULES] = {
   {OS                  , SORT_BY_HITS , SORT_DESC } ,
   {BROWSERS            , SORT_BY_HITS , SORT_DESC } ,
   {VISIT_TIMES         , SORT_BY_DATA , SORT_ASC  } ,
+  {VIRTUAL_HOSTS       , SORT_BY_HITS , SORT_DESC } ,
   {REFERRERS           , SORT_BY_HITS , SORT_DESC } ,
   {REFERRING_SITES     , SORT_BY_HITS , SORT_DESC } ,
   {KEYPHRASES          , SORT_BY_HITS , SORT_DESC } ,
-#ifdef HAVE_LIBGEOIP
+  {STATUS_CODES        , SORT_BY_HITS , SORT_DESC } ,
+  {REMOTE_USER         , SORT_BY_HITS , SORT_DESC } ,
+#ifdef HAVE_GEOLOCATION
   {GEO_LOCATION        , SORT_BY_HITS , SORT_DESC } ,
 #endif
-  {STATUS_CODES        , SORT_BY_HITS , SORT_DESC } ,
 };
 /* *INDENT-ON* */
 
-/* sort data ascending */
+/* Sort an array of strings ascending */
+int
+strcmp_asc (const void *a, const void *b)
+{
+  return strcmp (*((char **) a), *((char **) b));
+}
+
+/* Sort 'data' metric ascending */
 static int
 cmp_data_asc (const void *a, const void *b)
 {
@@ -98,7 +134,7 @@ cmp_data_asc (const void *a, const void *b)
   return strcmp (ia->metrics->data, ib->metrics->data);
 }
 
-/* sort data descending */
+/* Sort 'data' metric descending */
 static int
 cmp_data_desc (const void *a, const void *b)
 {
@@ -107,7 +143,7 @@ cmp_data_desc (const void *a, const void *b)
   return strcmp (ib->metrics->data, ia->metrics->data);
 }
 
-/* sort numeric descending */
+/* Sort 'hits' metric descending */
 static int
 cmp_num_desc (const void *a, const void *b)
 {
@@ -120,7 +156,7 @@ cmp_num_desc (const void *a, const void *b)
   return (va < vb) - (va > vb);
 }
 
-/* sort numeric ascending */
+/* Sort 'hits' metric ascending */
 static int
 cmp_num_asc (const void *a, const void *b)
 {
@@ -133,7 +169,7 @@ cmp_num_asc (const void *a, const void *b)
   return (va > vb) - (va < vb);
 }
 
-/* sort numeric descending */
+/* Sort 'visitors' metric descending */
 static int
 cmp_vis_desc (const void *a, const void *b)
 {
@@ -146,7 +182,7 @@ cmp_vis_desc (const void *a, const void *b)
   return (va < vb) - (va > vb);
 }
 
-/* sort numeric ascending */
+/* Sort 'visitors' metric ascending */
 static int
 cmp_vis_asc (const void *a, const void *b)
 {
@@ -159,23 +195,30 @@ cmp_vis_asc (const void *a, const void *b)
   return (va > vb) - (va < vb);
 }
 
-/* sort raw numeric descending */
+/* Sort GRawDataItem value descending */
 static int
 cmp_raw_num_desc (const void *a, const void *b)
 {
   const GRawDataItem *ia = a;
   const GRawDataItem *ib = b;
 
-  GDataMap *amap = ia->value;
-  GDataMap *bmap = ib->value;
-
-  int va = amap->data;
-  int vb = bmap->data;
+  int va = ia->value.ivalue;
+  int vb = ib->value.ivalue;
 
   return (va < vb) - (va > vb);
 }
 
-/* sort bandwidth descending */
+/* Sort GRawDataItem value descending */
+static int
+cmp_raw_str_desc (const void *a, const void *b)
+{
+  const GRawDataItem *ia = a;
+  const GRawDataItem *ib = b;
+
+  return strcmp (ib->value.svalue, ia->value.svalue);
+}
+
+/* Sort 'bandwidth' metric descending */
 static int
 cmp_bw_desc (const void *a, const void *b)
 {
@@ -188,7 +231,7 @@ cmp_bw_desc (const void *a, const void *b)
   return (va < vb) - (va > vb);
 }
 
-/* sort bandwidth ascending */
+/* Sort 'bandwidth' metric ascending */
 static int
 cmp_bw_asc (const void *a, const void *b)
 {
@@ -201,7 +244,7 @@ cmp_bw_asc (const void *a, const void *b)
   return (va > vb) - (va < vb);
 }
 
-/* sort usec descending */
+/* Sort 'avgts' metric descending */
 static int
 cmp_avgts_desc (const void *a, const void *b)
 {
@@ -214,7 +257,7 @@ cmp_avgts_desc (const void *a, const void *b)
   return (va < vb) - (va > vb);
 }
 
-/* sort usec ascending */
+/* Sort 'avgts' metric ascending */
 static int
 cmp_avgts_asc (const void *a, const void *b)
 {
@@ -227,7 +270,7 @@ cmp_avgts_asc (const void *a, const void *b)
   return (va > vb) - (va < vb);
 }
 
-/* sort usec descending */
+/* Sort 'cumts' metric descending */
 static int
 cmp_cumts_desc (const void *a, const void *b)
 {
@@ -240,7 +283,7 @@ cmp_cumts_desc (const void *a, const void *b)
   return (va < vb) - (va > vb);
 }
 
-/* sort usec ascending */
+/* Sort 'cumts' metric ascending */
 static int
 cmp_cumts_asc (const void *a, const void *b)
 {
@@ -253,7 +296,7 @@ cmp_cumts_asc (const void *a, const void *b)
   return (va > vb) - (va < vb);
 }
 
-/* sort usec descending */
+/* Sort 'maxts' metric descending */
 static int
 cmp_maxts_desc (const void *a, const void *b)
 {
@@ -266,7 +309,7 @@ cmp_maxts_desc (const void *a, const void *b)
   return (va < vb) - (va > vb);
 }
 
-/* sort usec ascending */
+/* Sort 'maxts' metric ascending */
 static int
 cmp_maxts_asc (const void *a, const void *b)
 {
@@ -279,7 +322,7 @@ cmp_maxts_asc (const void *a, const void *b)
   return (va > vb) - (va < vb);
 }
 
-/* sort protocol ascending */
+/* Sort 'protocol' metric ascending */
 static int
 cmp_proto_asc (const void *a, const void *b)
 {
@@ -288,7 +331,7 @@ cmp_proto_asc (const void *a, const void *b)
   return strcmp (ia->metrics->protocol, ib->metrics->protocol);
 }
 
-/* sort protocol descending */
+/* Sort 'protocol' metric descending */
 static int
 cmp_proto_desc (const void *a, const void *b)
 {
@@ -297,7 +340,7 @@ cmp_proto_desc (const void *a, const void *b)
   return strcmp (ib->metrics->protocol, ia->metrics->protocol);
 }
 
-/* sort method ascending */
+/* Sort 'method' metric ascending */
 static int
 cmp_mthd_asc (const void *a, const void *b)
 {
@@ -306,7 +349,7 @@ cmp_mthd_asc (const void *a, const void *b)
   return strcmp (ia->metrics->method, ib->metrics->method);
 }
 
-/* sort method descending */
+/* Sort 'method' metric descending */
 static int
 cmp_mthd_desc (const void *a, const void *b)
 {
@@ -315,18 +358,69 @@ cmp_mthd_desc (const void *a, const void *b)
   return strcmp (ib->metrics->method, ia->metrics->method);
 }
 
+/* Given a string sort field, get the enum field value.
+ *
+ * On error, -1 is returned.
+ * On success, the enumerated field value is returned. */
 int
 get_sort_field_enum (const char *str)
 {
   return str2enum (FIELD, ARRAY_SIZE (FIELD), str);
 }
 
+/* Given a string sort order, get the enum order value.
+ *
+ * On error, -1 is returned.
+ * On success, the enumerated order value is returned. */
 int
 get_sort_order_enum (const char *str)
 {
   return str2enum (ORDER, ARRAY_SIZE (ORDER), str);
 }
 
+/* Given a GSortOrder enum value, return the corresponding string.
+ *
+ * The string corresponding to the enumerated order value is returned. */
+const char *
+get_sort_order_str (GSortOrder order)
+{
+  return ORDER[order].str;
+}
+
+/* Given a GSortField enum value, return the corresponding string.
+ *
+ * The string corresponding to the enumerated field value is returned. */
+const char *
+get_sort_field_str (GSortField field)
+{
+  return FIELD[field].str;
+}
+
+/* Given a GSortField enum value, return the corresponding key.
+ *
+ * The key corresponding to the enumerated field value is returned. */
+const char *
+get_sort_field_key (GSortField field)
+{
+  static const char *field2key[][2] = {
+    {"BY_HITS", "hits"},
+    {"BY_VISITORS", "visitors"},
+    {"BY_DATA", "data"},
+    {"BY_BW", "bytes"},
+    {"BY_AVGTS", "avgts"},
+    {"BY_CUMTS", "cumts"},
+    {"BY_MAXTS", "maxts"},
+    {"BY_PROT", "protocol"},
+    {"BY_MTHD", "method"},
+  };
+
+  return field2key[field][1];
+}
+
+/* Set the initial metric sort per module/panel.
+ *
+ * On error, function returns.
+ * On success, panel metrics are sorted. */
 void
 set_initial_sort (const char *smod, const char *sfield, const char *ssort)
 {
@@ -345,6 +439,10 @@ set_initial_sort (const char *smod, const char *sfield, const char *ssort)
   module_sort[module].sort = order;
 }
 
+/* Determine if module/panel metric can be sorted.
+ *
+ * On error or if metric can't be sorted, 0 is returned.
+ * On success, 1 is returned. */
 int
 can_sort_module (GModule module, int field)
 {
@@ -372,21 +470,25 @@ can_sort_module (GModule module, int field)
   return can_sort;
 }
 
+/* Parse all initial sort options from the config file.
+ *
+ * On error, function returns.
+ * On success, panel metrics are sorted. */
 void
 parse_initial_sort (void)
 {
   int i;
-  char *view;
   char module[SORT_MODULE_LEN], field[SORT_FIELD_LEN], order[SORT_ORDER_LEN];
   for (i = 0; i < conf.sort_panel_idx; ++i) {
-    view = conf.sort_panels[i];
-    if (sscanf (view, "%15[^','],%11[^','],%4s", module, field, order) != 3)
+    if (sscanf
+        (conf.sort_panels[i], "%15[^','],%11[^','],%4s", module, field,
+         order) != 3)
       continue;
     set_initial_sort (module, field, order);
   }
 }
 
-/* apply user defined sort */
+/* Apply user defined sort */
 void
 sort_holder_items (GHolderItem * items, int size, GSort sort)
 {
@@ -448,10 +550,23 @@ sort_holder_items (GHolderItem * items, int size, GSort sort)
   }
 }
 
-/* sort raw data for the first run (default sort) */
+/* Sort raw numeric data in a descending order for the first run
+ * (default sort)
+ *
+ * On success, raw data sorted in a descending order. */
 GRawData *
-sort_raw_data (GRawData * raw, int ht_size)
+sort_raw_num_data (GRawData * raw_data, int ht_size)
 {
-  qsort (raw->items, ht_size, sizeof (GRawDataItem), cmp_raw_num_desc);
-  return raw;
+  qsort (raw_data->items, ht_size, sizeof *(raw_data->items), cmp_raw_num_desc);
+  return raw_data;
+}
+
+/* Sort raw string data in a descending order for the first run.
+ *
+ * On success, raw data sorted in a descending order. */
+GRawData *
+sort_raw_str_data (GRawData * raw_data, int ht_size)
+{
+  qsort (raw_data->items, ht_size, sizeof *(raw_data->items), cmp_raw_str_desc);
+  return raw_data;
 }
