@@ -1,22 +1,31 @@
 /**
  * util.c -- a set of handy functions to help parsing
- * Copyright (C) 2009-2014 by Gerardo Orellana <goaccess@prosoftcorp.com>
- * GoAccess - An Ncurses apache weblog analyzer & interactive viewer
+ *    ______      ___
+ *   / ____/___  /   | _____________  __________
+ *  / / __/ __ \/ /| |/ ___/ ___/ _ \/ ___/ ___/
+ * / /_/ / /_/ / ___ / /__/ /__/  __(__  |__  )
+ * \____/\____/_/  |_\___/\___/\___/____/____/
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License, or (at your option) any later version.
+ * The MIT License (MIT)
+ * Copyright (c) 2009-2016 Gerardo Orellana <hello @ goaccess.io>
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * A copy of the GNU General Public License is attached to this
- * source distribution for its full text.
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
  *
- * Visit http://goaccess.prosoftcorp.com for new releases.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #define _LARGEFILE_SOURCE
@@ -47,6 +56,7 @@
 #include "error.h"
 #include "xmalloc.h"
 
+/* HTTP status codes categories */
 static const char *code_type[][2] = {
   {"1", "1xx Informational"},
   {"2", "2xx Success"},
@@ -55,16 +65,19 @@ static const char *code_type[][2] = {
   {"5", "5xx Server Error"},
 };
 
+/* HTTP status codes */
 static const char *codes[][2] = {
   {"100", "100 - Continue: Server received the initial part of the request"},
   {"101", "101 - Switching Protocols: Client asked to switch protocols"},
   {"200", "200 - OK: The request sent by the client was successful"},
   {"201", "201 - Created: The request has been fulfilled and created"},
   {"202", "202 - Accepted: The request has been accepted for processing"},
-  {"203", "203 - Non-Authoritative Information: Response from a third party"},
+  {"203", "203 - Non-authoritative Information: Response from a third party"},
   {"204", "204 - No Content: Request did not return any content"},
   {"205", "205 - Reset Content: Server asked the client to reset the document"},
   {"206", "206 - Partial Content: The partial GET has been successful"},
+  {"207", "207 - Multi-Status: WebDAV; RFC 4918"},
+  {"208", "208 - Already Reported: WebDAV; RFC 5842"},
   {"300", "300 - Multiple Choices: Multiple options for the resource"},
   {"301", "301 - Moved Permanently: Resource has permanently moved"},
   {"302", "302 - Moved Temporarily (redirect)"},
@@ -76,7 +89,7 @@ static const char *codes[][2] = {
   {"401", "401 - Unauthorized: Request needs user authentication"},
   {"402", "402 - Payment Required"},
   {"403", "403 - Forbidden: Server is refusing to respond to it"},
-  {"404", "404 - Document Not Found: Requested resource could not be found"},
+  {"404", "404 - Not Found: Requested resource could not be found"},
   {"405", "405 - Method Not Allowed: Request method not supported"},
   {"406", "406 - Not Acceptable"},
   {"407", "407 - Proxy Authentication Required"},
@@ -85,8 +98,8 @@ static const char *codes[][2] = {
   {"410", "410 - Gone: Resource requested is no longer available"},
   {"411", "411 - Length Required: Invalid Content-Length"},
   {"412", "412 - Precondition Failed: Server does not meet preconditions"},
-  {"413", "413 - Requested Entity Too Long"},
-  {"414", "414 - Requested Filename Too Long"},
+  {"413", "413 - Payload Too Large"},
+  {"414", "414 - Request-URI Too Long"},
   {"415", "415 - Unsupported Media Type: Media type is not supported"},
   {"416", "416 - Requested Range Not Satisfiable: Cannot supply that portion"},
   {"417", "417 - Expectation Failed"},
@@ -109,7 +122,10 @@ static const char *codes[][2] = {
   {"524", "524 - CloudFlare - A timeout occured"}
 };
 
-/* helper functions */
+/* Return part of a string
+ *
+ * On error NULL is returned.
+ * On success the extracted part of string is returned */
 char *
 substring (const char *str, int begin, int len)
 {
@@ -134,6 +150,9 @@ substring (const char *str, int begin, int len)
   return buffer;
 }
 
+/* A pointer to the allocated memory of the new string
+ *
+ * On success, a pointer to a new string is returned */
 char *
 alloc_string (const char *str)
 {
@@ -142,6 +161,8 @@ alloc_string (const char *str)
   return new;
 }
 
+/* A wrapper function to copy the first num characters of source to
+ * destination. */
 void
 xstrncpy (char *dest, const char *source, const size_t dest_size)
 {
@@ -153,6 +174,10 @@ xstrncpy (char *dest, const char *source, const size_t dest_size)
   }
 }
 
+/* Count the number of matches on the string `s1` given a character `c`
+ *
+ * If the character is not found, 0 is returned
+ * On success, the number of characters found */
 int
 count_matches (const char *s1, char c)
 {
@@ -165,6 +190,39 @@ count_matches (const char *s1, char c)
   return n;
 }
 
+/* String matching where one string contains wildcard characters.
+ *
+ * If no match found, 1 is returned.
+ * If match found, 0 is returned. */
+static int
+wc_match (const char *wc, char *str)
+{
+  while (*wc && *str) {
+    if (*wc == '*') {
+      while (*wc && *wc == '*')
+        wc++;
+      if (!*wc)
+        return 1;
+
+      while (*str && *str != *wc)
+        str++;
+    } else if (*wc == '?' || *wc == *str) {
+      wc++;
+      str++;
+    } else {
+      break;
+    }
+  }
+  if (!*wc && !*str)
+    return 1;
+  return 0;
+}
+
+/* Determine if the given host needs to be ignored given the list of
+ * referrers to ignore.
+ *
+ * On error, or the referrer is not found, 0 is returned
+ * On success, or if the host needs to be ignored, 1 is returned */
 int
 ignore_referer (const char *host)
 {
@@ -192,6 +250,10 @@ out:
   return ignore;
 }
 
+/* Determine if the given ip is within a range of IPs.
+ *
+ * On error, or not within the range, 0 is returned
+ * On success, or if within the range, 1 is returned */
 static int
 within_range (const char *ip, const char *start, const char *end)
 {
@@ -229,6 +291,11 @@ within_range (const char *ip, const char *start, const char *end)
   return 0;
 }
 
+/* Determine if the given IP needs to be ignored given the list of IPs
+ * to ignore.
+ *
+ * On error, or not within the range, 0 is returned
+ * On success, or if within the range, 1 is returned */
 int
 ip_in_range (const char *ip)
 {
@@ -267,7 +334,38 @@ ip_in_range (const char *ip)
   return 0;
 }
 
+/* Searches the array of output formats for the given extension value.
+ *
+ * If not found, 1 is returned.
+ * On success, the given filename is malloc'd and assigned and 0 is
+ * returned. */
+int
+find_output_type (char **filename, const char *ext, int alloc)
+{
+  int i;
+  const char *dot = NULL;
 
+  for (i = 0; i < conf.output_format_idx; ++i) {
+    /* for backwards compatibility. i.e., -o json  */
+    if (strcmp (conf.output_formats[i], ext) == 0)
+      return 0;
+
+    if ((dot = strrchr (conf.output_formats[i], '.')) != NULL &&
+        strcmp (dot + 1, ext) == 0) {
+      if (alloc)
+        *filename = xstrdup (conf.output_formats[i]);
+      return 0;
+    }
+  }
+
+  return 1;
+}
+
+/* Search the environment HOME variable and append GoAccess' config
+ * file.
+ *
+ * On error, it outputs an error message and the program terminates.
+ * On success, the path of HOME and the config file is returned. */
 char *
 get_home (void)
 {
@@ -283,6 +381,9 @@ get_home (void)
   return path;
 }
 
+/* Get the path to the global config file.
+ *
+ * On success, the path of the global config file is returned. */
 char *
 get_global_config (void)
 {
@@ -294,6 +395,28 @@ get_global_config (void)
   return path;
 }
 
+/* A self-checking wrapper to convert_date().
+ *
+ * On error, a newly malloc'd '---' string is returned.
+ * On success, a malloc'd 'Ymd' date is returned. */
+char *
+get_visitors_date (const char *odate, const char *from, const char *to)
+{
+  char date[DATE_TIME] = "";    /* Ymd */
+
+  memset (date, 0, sizeof *date);
+  /* verify we have a valid date conversion */
+  if (convert_date (date, odate, from, to, DATE_TIME) == 0)
+    return xstrdup (date);
+
+  LOG_DEBUG (("invalid date: %s", odate));
+  return xstrdup ("---");
+}
+
+/* Format the given date/time according the given format.
+ *
+ * On error, 1 is returned.
+ * On success, 0 is returned. */
 #pragma GCC diagnostic ignored "-Wformat-nonliteral"
 int
 str_to_time (const char *str, const char *fmt, struct tm *tm)
@@ -304,7 +427,7 @@ str_to_time (const char *str, const char *fmt, struct tm *tm)
   if (str == NULL || *str == '\0' || fmt == NULL || *fmt == '\0')
     return 1;
 
-  /* check if char string needs to be convert from microseconds */
+  /* check if char string needs to be converted from microseconds */
   if (strcmp ("%f", fmt) == 0) {
     errno = 0;
     tm->tm_year = 1970 - 1900;
@@ -329,8 +452,13 @@ str_to_time (const char *str, const char *fmt, struct tm *tm)
   return 0;
 }
 
+/* Convert a date from one format to another and store inot the given buffer.
+ *
+ * On error, 1 is returned.
+ * On success, 0 is returned. */
 int
-convert_date (char *res, char *data, const char *from, const char *to, int size)
+convert_date (char *res, const char *data, const char *from, const char *to,
+              int size)
 {
   struct tm tm;
 
@@ -353,6 +481,10 @@ convert_date (char *res, char *data, const char *from, const char *to, int size)
 
 #pragma GCC diagnostic warning "-Wformat-nonliteral"
 
+/* Determine if the given IP is a valid IPv4/IPv6 address.
+ *
+ * On error, 1 is returned.
+ * On success, 0 is returned. */
 int
 invalid_ipaddr (char *str, int *ipvx)
 {
@@ -379,7 +511,10 @@ invalid_ipaddr (char *str, int *ipvx)
   return 1;
 }
 
-/* off_t becomes 64 bit aware */
+/* Get information about the filename.
+ *
+ * On error, -1 is returned.
+ * On success, the file size of the given filename. */
 off_t
 file_size (const char *filename)
 {
@@ -393,6 +528,11 @@ file_size (const char *filename)
   return -1;
 }
 
+/* Determine if the given status code is within the list of status
+ * codes and find out the status type/category.
+ *
+ * If not found, "Unknown" is returned.
+ * On success, the status code type/category is returned. */
 const char *
 verify_status_code_type (const char *str)
 {
@@ -404,6 +544,11 @@ verify_status_code_type (const char *str)
   return "Unknown";
 }
 
+/* Determine if the given status code is within the list of status
+ * codes.
+ *
+ * If not found, "Unknown" is returned.
+ * On success, the status code is returned. */
 const char *
 verify_status_code (char *str)
 {
@@ -415,6 +560,25 @@ verify_status_code (char *str)
   return "Unknown";
 }
 
+/* Checks if the given string is within the given array.
+ *
+ * If not found, -1 is returned.
+ * If found, the key for needle in the array is returned. */
+int
+str_inarray (const char *s, const char *arr[], int size)
+{
+  int i;
+  for (i = 0; i < size; i++) {
+    if (strcmp (arr[i], s) == 0)
+      return i;
+  }
+  return -1;
+}
+
+/* Strip whitespace from the beginning of a string.
+ *
+ * On success, a string with whitespace stripped from the beginning of
+ * the string is returned. */
 char *
 ltrim (char *s)
 {
@@ -427,6 +591,10 @@ ltrim (char *s)
   return s;
 }
 
+/* Strip whitespace from the end of a string.
+ *
+ * On success, a string with whitespace stripped from the end of the
+ * string is returned. */
 char *
 rtrim (char *s)
 {
@@ -439,32 +607,45 @@ rtrim (char *s)
   return s;
 }
 
+/* Strip whitespace from the beginning and end of the string.
+ *
+ * On success, the trimmed string is returned. */
 char *
 trim_str (char *str)
 {
   return rtrim (ltrim (str));
 }
 
+/* Convert the file size in bytes to a human readable format.
+ *
+ * On error, the original size of the string in bytes is returned.
+ * On success, the file size in a human readable format is returned. */
 char *
 filesize_str (unsigned long long log_size)
 {
-  char *size = (char *) xmalloc (sizeof (char) * 12);
-  if (log_size >= GB)
-    snprintf (size, 12, "%.2f GiB", (double) (log_size) / GB);
-  else if (log_size >= MB)
-    snprintf (size, 12, "%.2f MiB", (double) (log_size) / MB);
-  else if (log_size >= KB)
-    snprintf (size, 12, "%.2f KiB", (double) (log_size) / KB);
+  char *size = xmalloc (sizeof (char) * 12);
+  if (log_size >= TIB)
+    snprintf (size, 12, "%.2f TiB", (double) (log_size) / TIB);
+  else if (log_size >= GIB)
+    snprintf (size, 12, "%.2f GiB", (double) (log_size) / GIB);
+  else if (log_size >= MIB)
+    snprintf (size, 12, "%.2f MiB", (double) (log_size) / MIB);
+  else if (log_size >= KIB)
+    snprintf (size, 12, "%.2f KiB", (double) (log_size) / KIB);
   else
     snprintf (size, 12, "%.1f   B", (double) (log_size));
 
   return size;
 }
 
+/* Convert microseconds to a human readable format.
+ *
+ * On error, a malloc'd string in microseconds is returned.
+ * On success, the time in a human readable format is returned. */
 char *
 usecs_to_str (unsigned long long usec)
 {
-  char *size = (char *) xmalloc (sizeof (char) * 11);
+  char *size = xmalloc (sizeof (char) * 11);
   if (usec >= DAY)
     snprintf (size, 11, "%.2f  d", (double) (usec) / DAY);
   else if (usec >= HOUR)
@@ -481,6 +662,10 @@ usecs_to_str (unsigned long long usec)
   return size;
 }
 
+/* Convert the given int to a string with the ability to add some
+ * padding.
+ *
+ * On success, the given number as a string is returned. */
 char *
 int2str (int d, int width)
 {
@@ -490,15 +675,10 @@ int2str (int d, int width)
   return s;
 }
 
-char *
-ints_to_str (int a, int b)
-{
-  char *s = xmalloc (snprintf (NULL, 0, "%d%d", a, b) + 1);
-  sprintf (s, "%d%d", a, b);
-
-  return s;
-}
-
+/* Convert the given float to a string with the ability to add some
+ * padding.
+ *
+ * On success, the given number as a string is returned. */
 char *
 float2str (float d, int width)
 {
@@ -508,6 +688,9 @@ float2str (float d, int width)
   return s;
 }
 
+/* Determine the length of an integer (number of digits).
+ *
+ * On success, the length of the number is returned. */
 int
 intlen (int num)
 {
@@ -520,6 +703,9 @@ intlen (int num)
   return l;
 }
 
+/* Allocate a new string and fill it with the given character.
+ *
+ * On success, the newly allocated string is returned. */
 char *
 char_repeat (int n, char c)
 {
@@ -530,7 +716,11 @@ char_repeat (int n, char c)
   return dest;
 }
 
-/* replace old with new char */
+/* Replace all occurrences of the given char with the replacement
+ * char.
+ *
+ * On error the original string is returned.
+ * On success, a string with the replaced values is returned. */
 char *
 char_replace (char *str, char o, char n)
 {
@@ -545,6 +735,9 @@ char_replace (char *str, char o, char n)
   return str;
 }
 
+/* Remove all occurrences of a new line.
+ *
+ * On success, a string with the replaced new lines is returned. */
 void
 strip_newlines (char *str)
 {
@@ -557,7 +750,9 @@ strip_newlines (char *str)
   *dst = '\0';
 }
 
-/* strip blanks from a string */
+/* Strip blanks from a string.
+ *
+ * On success, a string without whitespace is returned. */
 char *
 deblank (char *str)
 {
@@ -572,6 +767,10 @@ deblank (char *str)
   return out;
 }
 
+/* Make a string uppercase.
+ *
+ * On error the original string is returned.
+ * On success, the uppercased string is returned. */
 char *
 strtoupper (char *str)
 {
@@ -587,6 +786,9 @@ strtoupper (char *str)
   return str;
 }
 
+/* Left-pad a string with n amount of spaces.
+ *
+ * On success, a left-padded string is returned. */
 char *
 left_pad_str (const char *s, int indent)
 {
@@ -599,74 +801,25 @@ left_pad_str (const char *s, int indent)
   return buf;
 }
 
-int
-wc_match (char *wc, char *str)
+/* Append the source string to destination and reallocates and
+ * updating the destination buffer appropriately. */
+void
+append_str (char **dest, const char *src)
 {
-  while (*wc && *str) {
-    if (*wc == '*') {
-      while (*wc && *wc == '*')
-        wc++;
-      if (!*wc)
-        return 1;
+  size_t curlen = strlen (*dest);
+  size_t srclen = strlen (src);
+  size_t newlen = curlen + srclen;
 
-      while (*str && *str != *wc)
-        str++;
-    } else if (*wc == '?' || *wc == *str) {
-      wc++;
-      str++;
-    } else {
-      break;
-    }
-  }
-  if (!*wc && !*str)
-    return 1;
-  return 0;
+  char *str = xrealloc (*dest, newlen + 1);
+  memcpy (str + curlen, src, srclen + 1);
+  *dest = str;
 }
 
-/* returns unescaped malloc'd string */
-char *
-unescape_str (const char *src)
-{
-  char *dest, *q;
-  const char *p = src;
-
-  if (src == NULL || *src == '\0')
-    return NULL;
-
-  dest = xmalloc (strlen (src) + 1);
-  q = dest;
-
-  while (*p) {
-    if (*p == '\\') {
-      p++;
-      switch (*p) {
-      case '\0':
-        /* warning... */
-        goto out;
-      case 'n':
-        *q++ = '\n';
-        break;
-      case 'r':
-        *q++ = '\r';
-        break;
-      case 't':
-        *q++ = '\t';
-        break;
-      default:
-        *q++ = *p;
-        break;
-      }
-    } else
-      *q++ = *p;
-    p++;
-  }
-out:
-  *q = 0;
-
-  return dest;
-}
-
-/* returns escaped malloc'd string */
+/* Escapes the special characters, e.g., '\n', '\r', '\t', '\'
+ * in the string source by inserting a '\' before them.
+ *
+ * On error NULL is returned.
+ * On success the escaped string is returned */
 char *
 escape_str (const char *src)
 {
@@ -713,3 +866,49 @@ escape_str (const char *src)
   *q = 0;
   return dest;
 }
+
+/* Get an unescaped malloc'd string
+ *
+ * On error NULL is returned.
+ * On success the unescaped string is returned */
+char *
+unescape_str (const char *src)
+{
+  char *dest, *q;
+  const char *p = src;
+
+  if (src == NULL || *src == '\0')
+    return NULL;
+
+  dest = xmalloc (strlen (src) + 1);
+  q = dest;
+
+  while (*p) {
+    if (*p == '\\') {
+      p++;
+      switch (*p) {
+      case '\0':
+        /* warning... */
+        goto out;
+      case 'n':
+        *q++ = '\n';
+        break;
+      case 'r':
+        *q++ = '\r';
+        break;
+      case 't':
+        *q++ = '\t';
+        break;
+      default:
+        *q++ = *p;
+        break;
+      }
+    } else
+      *q++ = *p;
+    p++;
+  }
+out:
+  *q = 0;
+
+  return dest;
+}
